---
Date: 2022-09-01
Author: Alexandre LUCAZEAU
Title: Nixos - développer une application python flask avec nix-shell
Slug: 
categories: Nixos
Tags: 
- nixos
- nix-shell
- python
- flask
---
# nix-shell et dotenv
Pour mes besoins de gestion, je souhaite me développer de quoi alimenter une base de données PostgreSQL. Rien de très compliqué, mais qui va me faire gagner du temps et me permettre d'assurer le pilotage de mes affaires.

Je suis pas développeur, je connais rien aux css et et pas plus au js.

Mais je sais de quoi j'ai besoins.
Il y a longtemps que j'aurai du me mettre à faire un peu de dev. Je vais pas devenir un ténor, mais au moins je serai capable d'avancer en autonomie.

Par ou commencer ? Quel language est le plus simple à prendre en main, pour aller à l'essentiel (le service que va me rendre l'application) et une bonne intégration/usage avec mon poste de travail ?

Après une rapide étude et quelques tests :
* Php : certainement très bien, mais php n'est pas trop utilisé en dehors du web, et j'accroche pas à son fonctionnement.
* Ruby : superbe language, mais les gem c'est toujours la merde. J'ai voulu installer la gem rails et paff ça plante sur la compilation d'un module. Classique mais pénible.
* Python : ça ressemble à une piste valable. J'ai tjs lutter contre mais faut pas être c.. on va donc essayer. Plus particulièrement, il y a le framework Flask qui est réputé simple et rapide à prendre en mains.

# Mise en place des briques

Au quotidien, j'utilise **fish** comme shell, mais il y a quelques problèmes car le standard est bash. Ici le vrai problème que j'ai rencontré, c'est dans l'utilisation du serveur http intégré de Flask. Avec fish, il plante, avec bash tout va bien.

On va donc faire en sorte de lancer le serveur de développement, en étant sous bash.

## Création d'un environnement de développement

La solution miracle se nomme nix-shell. Dans ce qui va devenir notre répertoire de développement, on crée le fichier **default.nix** suivant :

    with import<nixpkgs> {};
    stdenv.mkDerivation rec {
      name = "env";
      env = buildEnv { name = name; paths = buildInputs; };
      buildInputs = [
        python3
        python39Packages.flask
      ];
    }

Dans ce fichier on crée une dérivation nommée **rec** constituées des paquets python3 et python39Packages.flask.
La dérivation va nous créer un environnement avec python39 et flask. On peut y ajouter tout ce dont on a besoins, comme le moteur de la bdd PostgreSQL (ce qu'on fera plustard dans le projet)

Comme je suis faignant, je vais utiliser un fichier **.envrc** pour charger automatiquement mon nix-shell. On ajoute donc la ligne suivante dans le fichier **.envrc** :

    use_nix

On oubli pas d'autoriser le chargement du fichier **.envrc** via la commande :

    direnv allow


Vous devriez maintenant avoir l'environnement de dispo

## Création et chargement de l'environnement

La création de l'environnement virtuel python est classique :

    python -m venv ~/virtualenv/.flask-app

Pour le charger, il suffit de faire, avec Fish :

    source ~/virtualenv/.flask-app/bin/activate.fish

Avec Bash :

    source ~/virtualenv/.flask-app/bin/activate

A partir de maintenant vous pouvez commencer à utiliser pip et flask

## On peut pas avoir un chargement automatique ?

Par exemple, on entre dans le dossier et paf, ça charge le virtualenv, et on met à jour les modules via pip ?

Tout d'abord, si comme moi vous utilisez **fish** il faut s'assurer d'avoir dans le fichier **~/.config/fish/config.fish** la ligne suivante :

    direnv hook fish | source


Si vous utilisez bash, vous ajoutez à votre **.bashrc** :

    eval "$(direnv hook bash)"


ligne est nécessaire pour charger automatiquement le script **activate.fish**. Sans cette ligne vous aurez une erreur 2.


Notre fichier **default.nix** ressemble maintenant à ça :

	with import<nixpkgs> {};
	stdenv.mkDerivation rec {
	  name = "env";
	  env = buildEnv { name = name; paths = buildInputs; };
	  venvDir = "./.venv";
	  buildInputs = [
	    python3
	    python39Packages.flask
	  ];
	  shellHook = ''
	    source "${venvDir}/bin/activate"
	    #pip install -r requirements.txt
        export FLASK_APP=hello
        export FLASK_ENV=developpement
	    '';
	}

Normalement, dès que vous entrez dans le dossier, le virtualenv est chargé, les variables d'environnement aussi et vous pouvez tester votre nouvelle application avec un simple :

    flask run

Si vous avez besoins de modules via pip, il suffit de décommenter la ligne commençant par pip et de renseigner le fichier requirements.txt. Les modules seront automatiquement installés.

Reste plus qu'a mettre l'ensemble dans un dépôt git et à vous concentrer sur votre application.
